--[[
Scrolling input display Lua script
uses the Lua gd library (http://luaforge.net/projects/lua-gd/)
written by Dammit (dammit9x at hotmail dot com)
version 6/4/2010

Works with FBA, pcsx, snes9x and Gens:
http://code.google.com/p/fbarr/downloads/list
http://code.google.com/p/pcsxrr/downloads/list
http://code.google.com/p/snes9x-rr/downloads/list
http://code.google.com/p/gens-rerecording/downloads/list

You may tweak the parameters above the dashed line.
Positioning is optimized for CPS1-2-3. The x[2] value must be reduced for the other emus.

Possible improvements:
set up for games with other button configurations besides Capcom 6-button fighters
save and load data in savestates: not all emus have savestate.registersave/load lua functions
add/test other emus
]]

local imageset = "sf4"                   --the prefix that must be on image files
local switch = { [1] = "Q", [2] = "W" }  --key pressed to toggle each player on/off
local clearkey = "tilde"                 --key pressed to clear screen
local recordkey = "quote"                --key pressed to start/stop recording video
local recordpath = "framedump"           --folder to place recordings (relative to this lua file)
local opacity = 1.0                      --factor by which to multiply the opacity of the images
local timeout = 240                      --how many idle frames until old records are cleared
local buffersize = 10                    --how many records to show
local x = { [1] = 0x00, [2] = 0x120 }    --horizontal starting position
local y = 0x20                           --vertical starting position
local dx, dy = 0x10, 0x10                --how far apart to space the symbols

----------------------------------------------------------------------------------------------------
require "gd"
local thisframe, lastframe, oldswitch, e, keyset, ghetto, changed = {}, {}, {}
local recording, start, oldrecordkey, oldclearkey = false
local draw = { [1] = true, [2] = true }
local inp = { [1] = {}, [2] = {} }
local idle = { [1] = 0, [2] = 0 }
local emulator = { snes9x, gens, pcsx, fba }
local gamekeys = {
	"ul", "ur", "dl", "dr",
	{  "l",   "left",  "left",     "left",         "Left" }, 
	{  "r",  "right", "right",    "right",        "Right" }, 
	{  "u",     "up",    "up",       "up",           "Up" }, 
	{  "d",   "down",  "down",     "down",         "Down" }, 
	{ "LP",      "Y",     "X",   "square",   "Weak Punch" }, 
	{ "MP",      "X",     "Y", "triangle", "Medium Punch" }, 
	{ "HP",      "L",     "Z",       "r1", "Strong Punch" }, 
	{ "LK",      "B",     "A",        "x",    "Weak Kick" }, 
	{ "MK",      "A",     "B",   "circle",  "Medium Kick" }, 
	{ "HK",      "R",     "C",       "r2",  "Strong Kick" }, 
	{  "S", "select",  "none",    "start",        "Start" }
}

for k, emu in pairs(emulator) do --Detect what emulator this is.
	if emu then
		keyset = k+1
		e = emu
		break
	end
	error("This script doesn't work on this emulator yet.")
end

for index, name in ipairs(gamekeys) do --Check if all the images are present.
	local file = imageset .. (name[1] or name) .. ".png"
	if not io.open(file, "rb") then
		print(file, "not found.")
		ghetto = true
	end
end
if ghetto then print("Falling back on ghetto mode.") end

print("Scrolling input display Lua script, 6/4/2010.")
print("Press '" .. switch[1] .. "' to turn off/on P1 display.")
print("Press '" .. switch[2] .. "' to turn off/on P2 display.")
print("Press '" .. recordkey .. "' to start/stop recording to '" .. recordpath .. "' folder")
print()

local function filterinput(p, f)
	for pressed, state in pairs(joypad.getdown(p)) do --Check current controller state >
		for index, name in pairs(gamekeys) do           --but ignore non-gameplay buttons.
			if pressed == name[keyset] or pressed=="P" .. p .. " " .. tostring(name[keyset]) then
				f[name[1]] = state --FBA does not distinguish joypads, so inputs must be filtered by "P1" and "P2".
				break
			end
		end
	end
end

local function compositeinput(f)
	for _,a in pairs({"u", "d"}) do --Convert individual directions to diagonals.
		for _,b in pairs({"l", "r"}) do
			if f[a] and f[b] then
				f[a], f[b] = nil, nil
				f[a..b] = true
			end
		end
	end
end

local function detectchanges(f1, f2)
	changed = false
	for key, state in pairs(f2) do --If a key is pressed >
		if not f1[key] then          --that wasn't pressed last frame >
			changed = true             --then changes were made.
		end
	end
end

local function updaterecords(p, f2, i)
	if changed then                       --If changes were made >
		if idle[p] < timeout then           --and the player hasn't been idle too long >
			for record = buffersize, 2, -1 do
				i[record] = i[record-1]         --then shift every old record by 1 >
			end
		else
			for record = buffersize, 2, -1 do
				i[record] = nil                 --otherwise wipe out the old records.
			end
		end
		idle[p] = 0                         --Reset the idle count >
		i[1] = {}                           --and set current input as record 1 >
		local index = 1
		for _, name in ipairs(gamekeys) do  --but the order must not deviate from gamekeys.
			for key, state in pairs(f2) do
				if key == name[1] or key == name then
					i[1][index] = key
					index = index+1
					break
				end
			end
		end
	else idle[p] = idle[p]+1              --Increment the idle count if nothing changed.
	end
end

local function clear()
	local nowclearkey = input.get()[clearkey]
	if nowclearkey and not oldclearkey then
		inp = { [1] = {}, [2] = {} }
		print("Cleared screen.")
	end
	oldclearkey = nowclearkey
end

local function togglerecording()
	local nowrecordkey = input.get()[recordkey]
	if nowrecordkey and not oldrecordkey then
		recording = not recording
		if recording then
			start = movie.framecount()
			print("Started recording.")
		else
			local stop = movie.framecount()
			print("Stopped recording. (" .. stop - start .. " frames)")
			if stop > start then
				print("'moviedump-"..string.format("%06d",start)..".png' to 'moviedump-"..string.format("%06d",stop-1)..".png'")
			end
			print()
			start = nil
		end
	end
	oldrecordkey = nowrecordkey
end

local function toggleplayer(p)
	local nowswitch = input.get()[switch[p]] --Press the "switch" keys to toggle display for each char.
	if nowswitch and not oldswitch[p] then
		draw[p] = not draw[p]
		if draw[p] then print("player " .. p .. " on")
		else print("player " .. p .. " off")
		end
	end
	oldswitch[p] = nowswitch
end

local function displayrecords(p, i)
	if draw[p] then
		for line,stuff in pairs(i) do
			for index,key in pairs(i[line]) do
				if not ghetto then
					gui.gdoverlay(x[p]+index*dx, y+line*dy, gd.createFromPng(imageset .. key .. ".png"):gdStr(), opacity)
				else
					gui.text(x[p]+index*dx, y+line*dy, key) --Ghetto mode displays text symbols only.
				end
			end
		end
	end
end

e.registerbefore(function()
	for player = 1, 2 do
		thisframe = {}
		filterinput(player, thisframe)
		compositeinput(thisframe)
		detectchanges(lastframe[player], thisframe)
		updaterecords(player, thisframe, inp[player])
		lastframe[player] = thisframe
	end
	if recording then
		gd.createFromGdStr(gui.gdscreenshot()):png(recordpath .. "/moviedump-"..string.format("%06d",movie.framecount())..".png")
	end
end)

gui.register(function()
	gui.text(0,0,"")
	clear()
	togglerecording()
	for player = 1, 2 do
		toggleplayer(player)
		displayrecords(player, inp[player])
	end
end)